package com.gx.obe.business.enumeration;

/**
 * @Description: 待办枚举
 * @author chenxw
 */
public enum TaskEnum {
	
	PRE_EVALUATION("01", "com.gx.obe.common.action.PreEvaluationCompositeAction", PreEvaluation.class);
	
	public enum PreEvaluation implements TaskEnum.ITaskEnum {
		
		EXPERT_EEVIEW("01", "专家组长审核清标评议", "审核"),
		
		MANAGER_REVIEW("02", "项目经理审核清标评议", "审核");
		
		private String key;
		private String tsakName;
		private String operationName;
		
		PreEvaluation(String key, String tsakName, String operationName) {
			this.key = key;
			this.tsakName = tsakName;
			this.operationName = operationName;
			
		}
		
		@Override
		public String getKey() {
			return key;
		}
		
		@Override
		public String getTaskName() {
			return tsakName;
		}
		
		@Override
		public String getOperationName() {
			return operationName;
		}
		
	}
	
	public enum STATE {
		NOT_STARTED("00"),
		
		COMPLETE("01");
		
		private String key;
		
		STATE(String key) {
			this.key = key;
		}
		
		public String getKey() {
			return key;
		}
	}
	
	private String key;
	private String actionClassName;
	private ITaskEnum[] values;
	
	/**
	 * @author chenxw
	 * @param key
	 * @param actionClassName
	 * @param iTask
	 */
	private TaskEnum(String key, String actionClassName, Class<? extends ITaskEnum> iTask) {
		this.key = key;
		this.actionClassName = actionClassName;
		this.values = iTask.getEnumConstants();
	}
	
	/**
	 * @Description: 获取key
	 * @author chenxw
	 * @return
	 */
	public String getKey() {
		return key;
	}
	
	/**
	 * @Description: 获取action类名
	 * @author chenxw
	 * @return
	 */
	public String getActionClassName() {
		return actionClassName;
	}
	
	/**
	 * @Description: 获取下级枚举列表
	 * @author chenxw
	 * @return
	 */
	public ITaskEnum[] getValues() {
		return values;
	}
	
	public static String getActionClassName(String taskType) {
		for (TaskEnum taskEnum : TaskEnum.values()) {
			if (taskEnum.getKey().equals(taskType)) {
				return taskEnum.getActionClassName();
			}
		}
		return null;
	}
	
	/**
	 * @Description: 获取待办名
	 * @author chenxw
	 * @param taskType
	 * @param modeType
	 * @return
	 */
	public static String getTaskName(String taskType, String modeType) {
		for (TaskEnum taskEnum : TaskEnum.values()) {
			if (taskEnum.getKey().equals(taskType)) {
				for (ITaskEnum task : taskEnum.values) {
					if (task.getKey().equals(modeType)) {
						return task.getTaskName();
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 获取操作名
	 * @author chenxw
	 * @param taskType
	 * @param modeType
	 * @return
	 */
	public static String getOperationName(String taskType, String modeType) {
		for (TaskEnum taskEnum : TaskEnum.values()) {
			if (taskEnum.getKey().equals(taskType)) {
				for (ITaskEnum task : taskEnum.values) {
					if (task.getKey().equals(modeType)) {
						return task.getOperationName();
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 待办
	 * @author chenxw
	 */
	interface ITaskEnum {
		
		String getKey();
		
		String getTaskName();
		
		String getOperationName();
		
	}
	
}
