package com.gx.obe.business.http;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Calendar;
import java.util.Map.Entry;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;

import org.apache.log4j.Logger;

import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.MD5Utils;
import com.gx.utils.ActionLogUtils;

public class HttpUpload{
	public static final String ENCODING_REQUEST = "utf-8";
	public static final String ENCODING_RESPONSE = "utf-8";
	private Logger LOG = Logger.getLogger(HttpUpload.class);
	private String FileSize="FileSize";
	private String filemd5="filemd5";
	private String Range="Range";
	private String returnMd5 = "";
	private String operDesc = "";
	private boolean uploadSuccess = false;
	private File uploadFile;
	private HttpUrl httpUrl;
	private double uploadTotalSize = 0;
	private UploadListener uploadListener;
	public HttpUpload(UploadListener uploadListener){
		this.uploadListener = uploadListener; 
	}
	    
	/**
	 * @Description: 上传附件
	 * @author guoyr
	 * @param httpUrl
	 * @param uploadFile
	 * @return 
	 */
	public boolean upload(HttpUrl httpUrl, File uploadFile){
		uploadSuccess = false;
		try {
			this.httpUrl = httpUrl;
			this.uploadFile = uploadFile;
			httpUrl.setAttribute("stageIds", httpUrl.getAttribute("stageId"));
			httpUrl.setAttribute("packCodes", httpUrl.getAttribute("packCode"));
			if(null != uploadFile && uploadFile.exists()){
				long fileSize =  uploadFile.length();
				
				this.FileSize = String.valueOf(fileSize);
				try {
					this.filemd5 = MD5Utils.getFileMD5String(uploadFile);
				} catch (IOException e) {
				}//读取整个文件MD5值
				this.Range = "bytes=0-"+fileSize;
			}
			
			callBackMsg("正在上传文件，请等待...");
			long uploadStartTime = Calendar.getInstance().getTimeInMillis();
			this.upload(Range);
			LOG.error("文件["+uploadFile.getName()+"]"+Range+"上传结束用时:" + DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis() - uploadStartTime));
			if(uploadSuccess) {
				callBackMsg(true, "文件上传成功，等待服务器验证中...");
			}else {
				callBackMsg(false, "文件上传失败...");
			}
			// 告诉平台上传结束
			uploadStartTime = Calendar.getInstance().getTimeInMillis();
			this.upload("bytes=0-0");
			LOG.error("等待服务器验证用时:" + DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis() - uploadStartTime));
			if(uploadSuccess){
				if(null != returnMd5 && returnMd5.equals(filemd5)){
					callBackMsg(true, "上传验证成功");
				}else {
					callBackMsg(false, "上传成功，MD5值验证失败！");
					LOG.error("上传文件MD5为：" + filemd5 +"服务器接收到的md5为：" + returnMd5);
					uploadSuccess = false;
				}
			}else {
				callBackMsg(false, "上传失败，失败原因 ：" + (null != operDesc ? operDesc : ""));
			}
		} catch (Exception e) {
			uploadSuccess = false;
			callBackMsg(false, "上传失败，失败原因 ：" + (null != operDesc ? operDesc : ""));
			LOG.error("", e);
		}finally{
			ActionLogUtils.addServerLog(null, "上传" + uploadFile.getName(), null != httpUrl ? httpUrl.getUrl() + httpUrl.getAttributs() : "", "returnMd5:" +returnMd5 +"\toperDesc"+operDesc);
			
			callBackMsg("此次上传结束！");
		}
		return uploadSuccess;
	}
	
	public void upload(String range) {
		
		BufferedInputStream br = null;
		BufferedReader responseReader = null;
		
		try {
			URL urlgatewayUrl = new URL(httpUrl.getUrl() + httpUrl.getAttributs());
			SSLContext sslcontext = SSLContext.getInstance("SSL","SunJSSE");
			sslcontext.init(null ,  new TrustManager[]{new MyX509TrustManager()}, new java.security.SecureRandom());
//			URL url = new URL(urlPost);
			HostnameVerifier ignoreHostnameVerifier = new HostnameVerifier() {
				public boolean verify(String s, SSLSession sslsession) {
					System.out.println("WARNING: Hostname is not matched for cert.");
					return true;
				}
			};
			HttpsURLConnection.setDefaultSSLSocketFactory(sslcontext.getSocketFactory());
			HttpsURLConnection.setDefaultHostnameVerifier(ignoreHostnameVerifier);
			
			
			
			HttpURLConnection httpURLConnection = (HttpURLConnection) urlgatewayUrl.openConnection();
			int fileSize = (int) uploadFile.length();
			
//			int fileSize = (int) uploadFile.length();
			// 设置连接属性
			httpURLConnection.setDoOutput(true);
			httpURLConnection.setDoInput(true);
			httpURLConnection.setUseCaches(false);
			httpURLConnection.setConnectTimeout(60*60*1000);
			httpURLConnection.setReadTimeout(60*60*1000); 
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setRequestProperty("FileSize", FileSize);
			httpURLConnection.setRequestProperty("filemd5", filemd5);
			httpURLConnection.setRequestProperty("Range", range);
			httpURLConnection.setRequestProperty("Content-Type", "application/octet-stream");
			httpURLConnection.setRequestProperty("Content-length", "" + fileSize);
			
			if(null != httpUrl.getAttributesMap()){
				for(Entry<String, String> paramEntry : httpUrl.getAttributesMap().entrySet()){
					httpURLConnection.setRequestProperty(paramEntry.getKey(), paramEntry.getValue());
				}
			}
			if(!"bytes=0-0".equals(range)){
				br = new BufferedInputStream(new FileInputStream(uploadFile));
				int len = 0;
				byte[] bmff = new byte[4096]; // 建立缓冲区
				while ((len = br.read(bmff)) > 0) {
					httpURLConnection.getOutputStream().write(bmff, 0, len);
					uploadTotalSize += len;
				}
			}
			
			// 获得响应状态
			int responseCode = httpURLConnection.getResponseCode();
			if (HttpURLConnection.HTTP_OK == responseCode) {
				// 当正确响应时处理数据
				LOG.debug("Http Response OK...");
				// 处理响应流，必须与服务器响应流输出的编码一致
				StringBuffer responseBuffer = new StringBuffer();
				String readLine;
				
				responseReader = new BufferedReader(new InputStreamReader(httpURLConnection.getInputStream(), ENCODING_RESPONSE));
				while ((readLine = responseReader.readLine()) != null) {
					responseBuffer.append(readLine).append("n");
				}
				
				returnMd5 = httpURLConnection.getHeaderField("MD5");
				operDesc = httpURLConnection.getHeaderField("operDesc");
				uploadSuccess = null != httpURLConnection.getHeaderField("operTag") && httpURLConnection.getHeaderField("operTag").equals("Y");
				LOG.debug("Http Response:" + responseBuffer);
			}else {
				uploadSuccess = false;
				operDesc = "服务器请求异常...["+responseCode+"]";
				LOG.error(httpURLConnection.getHeaderFields());
			}
			LOG.debug("MD5："+httpURLConnection.getHeaderField("MD5"));
			LOG.debug("operTag："+httpURLConnection.getHeaderField("operTag"));
			LOG.debug(httpURLConnection.getHeaderFields());
			
		} catch (MalformedURLException e) {
			LOG.error("", e);
			operDesc = "服务器请求异常[URL]";
		} catch (IOException e) {
			operDesc = "服务器请求异常[IO]";
			LOG.error("", e);
		} catch (Exception e) {
			operDesc = "服务器请求异常...";
			LOG.error("", e);
		} finally {
			if(!uploadSuccess){
				LOG.error("上传失败！上传接口：" + httpUrl.getUrl() +"?" + httpUrl.getAttributs());
				LOG.error("returnMd5:" + returnMd5);
				LOG.error("operDesc:" + operDesc);
			}
			try {
				if(null != br){
					br.close();
				}
				if(null != responseReader){
					responseReader.close();
				}
			} catch (Exception e) {
				LOG.error("", e);
			}
		}
		
	}
	private void callBackMsg(String msg){
		if(null != uploadListener){
			uploadListener.info("  "+msg);
		}
	}
	private void callBackMsg(boolean bool, String msg){
		if(null != uploadListener){
			uploadListener.info((bool?"√":"×")+ msg);
		}
	}
}
