package com.gx.obe.business.http;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.util.utils.StringUtil;

import common.Logger;

public class HttpUrl {

	private String url;
	private String name;
	public static final String ENCODING_REQUEST = "utf-8";
	public static final String ENCODING_RESPONSE = "utf-8";
	private static Logger LOG = Logger.getLogger(HttpUrl.class);
	private Map<String, String> attributeMap;
	private String method = "POST"; 
	private String encoding = "UTF-8"; 
	private int connectTimeout = 100000;
	private int readTimeout = 1000000;
	private boolean showSuccessLog = false;
	private String exception;
	private HttpResponse httpResponse;
	private Map<String, String> requestPropertyMap;
	public HttpUrl(String url){
		this.url = url;
	}
	
	
	public String getName() {
		return name;
	}


	public void setName(String name) {
		this.name = name;
	}


	public int getConnectTimeout() {
		return connectTimeout;
	}


	public void setConnectTimeout(int connectTimeout) {
		this.connectTimeout = connectTimeout;
	}


	public int getReadTimeout() {
		return readTimeout;
	}


	public void setReadTimeout(int readTimeout) {
		this.readTimeout = readTimeout;
	}
	
	public boolean isShowSuccessLog() {
		return showSuccessLog;
	}


	public void setShowSuccessLog(boolean showSuccessLog) {
		this.showSuccessLog = showSuccessLog;
	}


	public String getUrl() {
		return url;
	}

	public void setUrl(String url) {
		this.url = url;
	}

	public String getMethod() {
		return method;
	}

	public void setMethod(String method) {
		this.method = method;
	}


	public Map<String, String> getAttributeMap() {
		return attributeMap;
	}


	public void setAttributeMap(Map<String, String> attributeMap) {
		this.attributeMap = attributeMap;
	}


	public String getEncoding() {
		return encoding;
	}


	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}

	public static String getEncodeStr(String value){
		if(PropertiesUtils.getProperty(ConfigConstants.URL_ENCODER, "00").equals("01")){
			if(null != value && value.length() > 0){
				try {
					return URLEncoder.encode(value, ENCODING_RESPONSE);
				} catch (UnsupportedEncodingException e) {
				}
			}
		}
		return value;
	}
	
	public void setAttributeEncode(String name, String value){
		if(PropertiesUtils.getProperty(ConfigConstants.URL_ENCODER, "00").equals("01")){
			try {
				setAttribute(name, null != value ? URLEncoder.encode(value, ENCODING_RESPONSE) : value);
			} catch (UnsupportedEncodingException e) {
				setAttribute(name, value);
			}
		}else {
			setAttribute(name, value);
		}
	}
	public void setAttribute(String name, String value){
		if(null != name){
			if(null == attributeMap){
				attributeMap = new HashMap<String, String>();
			}
			/***
			 * url出现了有+，空格，/，?，%，#，&，=等特殊符号的时候，可能在服务器端无法获得正确的参数值，如何是好？
					解决办法
					将这些字符转化成服务器可以识别的字符，对应关系如下：
					URL字符转义

					用其它字符替代吧，或用全角的。

					+    URL 中+号表示空格                                 %2B   
					空格 URL中的空格可以用+号或者编码           %20 
					/   分隔目录和子目录                                     %2F     
					?    分隔实际的URL和参数                             %3F     
					%    指定特殊字符                                          %25     
					#    表示书签                                                  %23     
					&    URL 中指定的参数间的分隔符                  %26     
					=    URL 中指定参数的值                                %3D
			 * 
			 */
//			StringBuilder testContent = new StringBuilder();
//			testContent.append("URL特殊字符测试\r\n");
//			testContent.append("+:表示空格\r\n");
//			testContent.append("/：分隔目录和子目录\r\n");
//			testContent.append("?:分隔实际的URL和参数\r\n");
//			testContent.append("%:指定特殊字符\r\n");
//			testContent.append("#:表示书签 \r\n");
//			testContent.append("&:URL 中指定的参数间的分隔符\r\n");
//			testContent.append("=:URL 中指定参数的值 \r\n");
					
			// 经测试发现% & = 不能在http参数中传递
			if(null != value && value.length() > 0){
				value = value.replace("%", "%25");
				value = value.replace("+", "%2B");
				value = value.replace(" ", "%20");
				value = value.replace("/", "%2F");
				value = value.replace("?", "%3F");
				value = value.replace("#", "%23");
				value = value.replace("&", "%26");
				value = value.replace("=", "%3D");
				attributeMap.put(name, value);
			}else {
				attributeMap.put(name, "");
			}
		}
	}
	
	/**
	 * 此方法不进行特殊字符替换（post方式可以不替换特殊字符）
	 * @param name
	 * @param value
	 */
	public void setAttributeTransference(String name, String value){
		if(null != name){
			if(null == attributeMap){
				attributeMap = new HashMap<String, String>();
			}
			if(null != value && value.length() > 0){
				attributeMap.put(name, value);
			}else {
				attributeMap.put(name, "");
			}
		}
	}
	
	public String getAttributs(){
		if(null != attributeMap && attributeMap.size() > 0){
			StringBuilder attributeSb = new StringBuilder();
			// 如果url中存在method
			if(url.indexOf("?") > 0){
				attributeSb.append("&");
			}else{
				attributeSb.append("?");
			}
			int index = 0;
			for(Entry<String, String> entry : attributeMap.entrySet()){
				if(index > 0){
					attributeSb.append("&");
				}
				attributeSb.append(entry.getKey()).append("=").append(entry.getValue());
				index ++;
			}
			return attributeSb.toString();
		}
		return "";
	}
	public String getAttribute(String key){
		if(null != attributeMap && null != attributeMap.get(key)){
			return attributeMap.get(key);
		}
		
		return null;
	}
	public Map<String, String> getAttributesMap(){
		return attributeMap;
	}
	


	public HttpResponse getHttpResponse() {
		return null != httpResponse ? httpResponse : new HttpResponse();
	}


	public void setHttpResponse(HttpResponse httpResponse) {
		this.httpResponse = httpResponse;
	}

	public Map<String, String> getRequestProperty() {
		return requestPropertyMap;
	}


	public void setRequestProperty(String property, String value) {
		if(null == requestPropertyMap) {
			requestPropertyMap = new HashMap<String, String>();
		} 
		requestPropertyMap.put(property, value);
	}

	public String getException() {
		return exception;
	}


	public void setException(String exception) {
		this.exception = exception;
	}


	public static void main(String[] arges) throws UnsupportedEncodingException{
		String value = "guoyongrong";
		value = "abc13579~";
		value = StringUtil.encodeString(value);
		System.out.println("encodeBase64:" + value);
		System.out.println(URLEncoder.encode(value, ENCODING_RESPONSE));
		value = "guoyongrong（郭永荣）";
		System.out.println(URLEncoder.encode(value, ENCODING_RESPONSE));
		
	}
}
