package com.gx.obe.business.http;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;

import org.apache.log4j.Logger;

import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.enumeration.UserEnum;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.obe.web.service.TransactionServerService;



/** 
 * @Description: 调用web服务工具类
 * @author guoyr 
 */
public class HttpUtils {
	
	private static TransactionServerService transactionServerService = new TransactionServerService();
	private static TenderProjectService tenderProjectService;
	private static int connectTimeout = 100000;
	private static int readTimeout = 1000000;
	private static Map<String, TransactionServer> transactionServerMap = new HashMap<String, TransactionServer>();
	private static Logger LOG = Logger.getLogger(HttpUtils.class);
	public static HttpUrl getProjectHttpUrl(TenderProjectEntity tenderProject, String invokeMethod){
		TransactionServer transactionServer = getTransactionServer(tenderProject.getTransactionServerId() );
		return getProjectHttpUrl(tenderProject, tenderProject.getParentProjectEntity(), transactionServer, invokeMethod);
	}
	public static HttpUrl getProjectHttpUrl(TenderProjectEntity tenderProject, TransactionServer transactionServer, String invokeMethod){
		return getProjectHttpUrl(tenderProject, tenderProject.getParentProjectEntity(), transactionServer, invokeMethod);
	}
	
	public static HttpUrl getProjectHttpUrl(TenderProjectEntity tenderProject, TenderProjectEntity parentProject, TransactionServer transactionServer, String invokeMethod){
		if(null == transactionServer){
			transactionServer = getTransactionServer(tenderProject.getTransactionServerId());
			if(null == transactionServer){
				return null;
			}
		}
		if(null == tenderProjectService){
			tenderProjectService = new TenderProjectService();
		}
		String prjCode = null;
		String packCode = null;
		if(null != parentProject){
			if(null != parentProject.getTenderNo()){
				prjCode = parentProject.getTenderNo();
			}else if(null != parentProject.getId()){
				parentProject = tenderProjectService.getById(parentProject.getId());
			}
		}else if(null != tenderProject.getParentId()){
			parentProject = tenderProjectService.getById(tenderProject.getParentId());
		}
		if(null != parentProject){
			prjCode = parentProject.getTenderNo();
			packCode = tenderProject.getTenderNo();
			if(null == tenderProject.getTenderNumber()){
				tenderProject.setTenderNumber(parentProject.getTenderNumber());
			}
		}else {
			prjCode = tenderProject.getTenderNo();
		}
		
		HttpUrl httpUrl = new HttpUrl(transactionServer.getServerAddress() + invokeMethod);
		httpUrl.setAttributeEncode("tenderNo", prjCode);
		httpUrl.setAttributeEncode("packCode", packCode);//现在平台暂不支持包件转码
		httpUrl.setAttribute("packCode", packCode);//现在平台暂不支持包件转码
		httpUrl.setAttribute("stageId", tenderProject.getStageId());
		boolean useUserInfo = false;
		if(UserEnum.USER_SOURCE_REMOTE.equals(Constants.USER.getSource()) && null != Constants.USER.getEncodePassWord()) {
			useUserInfo = true;
			if(!Constants.getPlatformUrl(null).equals(transactionServer.getServerAddress())) {
				LOG.error(tenderProject.getTenderNo() + tenderProject.getTenderName() + "不是当前平台"+Constants.getPlatformUrl(null)+"，项目来自："+transactionServer.getServerAddress());
				useUserInfo = false;
			}
		}
		if(useUserInfo) {
			httpUrl.setAttributeEncode("userName", Constants.USER.getUserAccount());
			httpUrl.setAttribute("password", Constants.USER.getEncodePassWord());
		}else {
			httpUrl.setAttributeEncode("userName", tenderProject.getServerAccount());
			httpUrl.setAttribute("password", tenderProject.getServerPassword());
		}
		httpUrl.setAttribute("tenderNoNumber", tenderProject.getTenderNumber());
		httpUrl.setAttribute("stageType", tenderProject.getPhaseType());
		
//		FileDownloadThread
		httpUrl.setAttribute("stageIds", tenderProject.getStageId());
		httpUrl.setAttribute("packCodes", packCode);
		httpUrl.setAttribute("accessType", "01");
		return httpUrl;
	}
	
	public static HttpResponse doPost(HttpUrl httpUrl){
		HttpResponse httpResponse = new HttpResponse();;
		if(null != httpUrl) {
			httpUrl.setHttpResponse(httpResponse);
			String returnValue = null;
			boolean flag = false;
			try {
				returnValue =  post(httpUrl.getUrl(), httpUrl.getAttributs(), httpUrl.getMethod(), httpUrl.getEncoding(), httpUrl.getConnectTimeout(), httpUrl.getReadTimeout(), httpUrl.getRequestProperty());
			} catch (Exception e1) {
				LOG.error(e1);
			}finally {
				httpResponse.setContent(returnValue);
				httpResponse.initHttpResponse();
				flag = httpResponse.getStatus();
				if(flag) {
					if(httpUrl.isShowSuccessLog()) {
						// 避免循环调用接口产生大量的日志
						LOG.info((null != httpUrl.getName() ? httpUrl.getName() : "接口请求") + (flag ? "成功" : "失败") + "\r\n接口："+httpUrl.getUrl() + "\t\n接口参数：" + httpUrl.getAttributs() +"\t\n:接口返回内容：" + httpUrl.getHttpResponse().getContent());
					}
				}else {
					LOG.error((null != httpUrl.getName() ? httpUrl.getName() : "接口请求") + (flag ? "成功" : "失败") + "\r\n接口："+httpUrl.getUrl() + "\t\n接口参数：" + httpUrl.getAttributs() +"\t\n:接口返回内容：" + httpUrl.getHttpResponse().getContent());
				}
				httpResponse = httpUrl.getHttpResponse();
			}
		}else {
			httpResponse.setMsg("不存在请求服务器！");
		}
		return httpResponse;
	}
	public static HttpResponse doPost(HttpUrl httpUrl, String params){
		HttpResponse httpResponse = new HttpResponse();;
		if(null != httpUrl) {
			httpUrl.setHttpResponse(httpResponse);
			String returnValue = null;
			boolean flag = false;
			try {
				returnValue =  post(httpUrl.getUrl(), params, httpUrl.getMethod(), httpUrl.getEncoding(), httpUrl.getConnectTimeout(), httpUrl.getReadTimeout(), httpUrl.getRequestProperty());
			} catch (Exception e1) {
				LOG.error(e1);
			}finally {
				httpResponse.setContent(returnValue);
				httpResponse.initHttpResponse();
				flag = httpResponse.getStatus();
				if(flag) {
					if(httpUrl.isShowSuccessLog()) {
						// 避免循环调用接口产生大量的日志
						LOG.info((null != httpUrl.getName() ? httpUrl.getName() : "接口请求") + (flag ? "成功" : "失败") + "\r\n接口："+httpUrl.getUrl() + "\t\n接口参数：" + params +"\t\n:接口返回内容：" + httpUrl.getHttpResponse().getContent());
					}
				}else {
					LOG.error((null != httpUrl.getName() ? httpUrl.getName() : "接口请求") + (flag ? "成功" : "失败") + "\r\n接口："+httpUrl.getUrl() + "\t\n接口参数：" + params +"\t\n:接口返回内容：" + httpUrl.getHttpResponse().getContent());
				}
				httpResponse = httpUrl.getHttpResponse();
			}
		}else {
			httpResponse.setMsg("不存在请求服务器！");
		}
		return httpResponse;
	}
	
	public static String post(String urlPost, String param, String method, String encoding, int connectTimeout, int readTimeout, Map<String, String> requestPropertyMap) throws Exception {
		
		SSLContext sslcontext= SSLContext.getInstance("SSL","SunJSSE");
		sslcontext.init(null ,  new TrustManager[]{new MyX509TrustManager()}, new java.security.SecureRandom());
//	URL url = new URL(urlPost);
		URL url = new URL(urlPost);
		HostnameVerifier ignoreHostnameVerifier = new HostnameVerifier() {
			public boolean verify(String s, SSLSession sslsession) {
				LOG.info("WARNING: Hostname is not matched for cert.");
				return true;
			}
		};
		HttpsURLConnection.setDefaultSSLSocketFactory(sslcontext.getSocketFactory());
		HttpsURLConnection.setDefaultHostnameVerifier(ignoreHostnameVerifier);
		
		HttpURLConnection http = (HttpURLConnection) url.openConnection();
		http.setDoOutput(true);
		http.setDoInput(true);
		http.setRequestMethod(method);
		http.setUseCaches(false);
		http.setConnectTimeout(connectTimeout);
		http.setReadTimeout(readTimeout);
		http.setInstanceFollowRedirects(true);
		http.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
		if(null != requestPropertyMap){
			for(Entry<String, String> propertyEntry : requestPropertyMap.entrySet()){
				if(null != propertyEntry.getValue()){
					http.setRequestProperty(propertyEntry.getKey(), propertyEntry.getValue());
				}
			}
		}
		
		http.connect();
		BufferedWriter out = new BufferedWriter(new OutputStreamWriter(http.getOutputStream(), encoding));
		if(null != param){
			out.write(param);
		}
		out.flush();
		out.close();
		BufferedReader in = new BufferedReader(new InputStreamReader(http.getInputStream(), encoding));
		String line = null;
		StringBuilder sb = new StringBuilder();
		while ((line = in.readLine()) != null) {
			sb.append(line);
		}
		in.close();
		http.disconnect();
		return sb.toString();
	}
	
	public static TransactionServer getTransactionServer(String serverId) {
		if(null == serverId) {
			return null;
		}
		TransactionServer transactionServer = transactionServerMap.get(serverId);
		if(null == transactionServer) {
			transactionServer = transactionServerService.getById(serverId);
			if(null == transactionServer) {
				transactionServer = transactionServerService.getCurPlatformTransactionServer(PropertiesUtils.getProperty(ConfigConstants.SERVER_NAME, "当前平台"), Constants.getPlatformUrl(null));
			}
			if(null != transactionServer) {
				transactionServerMap.put(serverId, transactionServer);
			}
		}
		return transactionServer;
	}
	
	public static void main(String[] args) {
		StringBuilder urlwww = new StringBuilder("http://172.16.7.63:8088/jeecg/tenderProjectInfoHS.do?getEvaluationFactorList");
		StringBuilder sbddd = new StringBuilder("&userName=zhangyl&password=MTIzNDU2&tenderNo=zk0707001&packCodes=1&stageType=01&stageId=2c9087bf55c446290155c457b318000a");
		try {
			URL url = new URL(urlwww.toString());
			HttpURLConnection http = (HttpURLConnection) url.openConnection();
			http.setDoOutput(true);
			http.setDoInput(true);
			http.setRequestMethod("POST");
			http.setUseCaches(false);
			http.setConnectTimeout(10000);
			http.setReadTimeout(10000);
			http.setInstanceFollowRedirects(true);
			http.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
			http.connect();
			BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
					http.getOutputStream(), "UTF-8"));
			out.write(sbddd.toString());
			out.flush();
			out.close();
			BufferedReader in = new BufferedReader(new InputStreamReader(
					http.getInputStream(), "UTF-8"));
			String line = null;
			StringBuilder sb = new StringBuilder();
			while ((line = in.readLine()) != null) {
				sb.append(line);
			}
			in.close();
			http.disconnect();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
