package com.gx.obe.business.vo;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.web.entity.evaluation.EvaluationStep;

/**
 * @Description: 
 * @author guoyr
 */
/**
 * @preserve private
 */
public class MaterialPriceCheckResult {
	
	private String id;
	private String code;//材料编号
	private String name;//材料名称
	private String unit;//单位
	private BigDecimal quantity;//工程量
	private BigDecimal price;//单价
	private BigDecimal comparePrice;//定额价（比较从）
	private int flag;// 过高还是过低（-1为过低；0为正常；1为过高）
	private BigDecimal difference;//差额
	private BigDecimal differenceRate;//差额率
	public static final BigDecimal MIN_DIFFERENCE_RATE = BigDecimal.valueOf(-25);// 最低差额率
	public static final BigDecimal MAX_DIFFERENCE_RATE = BigDecimal.valueOf(20);// 最高差额率
	private EvaluationStep evaluationStep;
	private MaterialPriceCheckResult parent;
	private String parentId;
	private List<MaterialPriceCheckResult> childList = new ArrayList<MaterialPriceCheckResult>();
	
	public MaterialPriceCheckResult(EvaluationStep evaluationStep) {
		this.evaluationStep = evaluationStep;
	}
	
	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	public String getCode() {
		return code;
	}
	
	public void setCode(String code) {
		this.code = code;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public String getUnit() {
		return unit;
	}
	
	public void setUnit(String unit) {
		this.unit = unit;
	}
	
	public BigDecimal getQuantity() {
		return quantity;
	}
	
	public void setQuantity(BigDecimal quantity) {
		this.quantity = quantity;
	}
	
	public int getFlag() {
		flag = 0;
		if (getDifferenceRate().compareTo(BigDecimal.ZERO) > 0) {
			BigDecimal priceRateMaxRate = evaluationStep.getEvaluationParamByKey(EvaluationStep.MAX_DIFFERENCE_RATE, BigDecimal.class);
			if (null == priceRateMaxRate || priceRateMaxRate.compareTo(BigDecimal.ZERO) <= 0) {
				priceRateMaxRate = BigDecimal.valueOf(25);
			}
			// 如果比最高差额率还高，则认为过高
			if (getDifferenceRate().compareTo(priceRateMaxRate) > 0) {
				flag = 1;
			}
		} else {
			BigDecimal priceRateMinRate = evaluationStep.getEvaluationParamByKey(EvaluationStep.MIN_DIFFERENCE_RATE, BigDecimal.class);
			if (null == priceRateMinRate || priceRateMinRate.compareTo(BigDecimal.ZERO) >= 0) {
				priceRateMinRate = BigDecimal.valueOf(-25);
			}
			// 如果比最低差额率还低，则认为过低
			if (getDifferenceRate().compareTo(priceRateMinRate) < 0) {
				flag = -1;
			}
		}
		return flag;
	}
	
	public void setFlag(int flag) {
		this.flag = flag;
	}
	
	public BigDecimal getDifference() {
		if (null != difference) {
			return difference;
		} else {
			difference = BigDecimalUtils.sub(getPrice(), getComparePrice());
			if (difference.compareTo(BigDecimal.ZERO) == 0) {
				difference = BigDecimal.ZERO;
			}
		}
		return difference;
	}
	
	public void setDifference(BigDecimal difference) {
		this.difference = difference;
	}
	
	public BigDecimal getDifferenceRate() {
		if (null != differenceRate) {
			return differenceRate;
		} else {
			differenceRate = BigDecimalUtils.mul(BigDecimalUtils.div(getDifference(), getComparePrice(), 4), BigDecimal.valueOf(100));
			if (differenceRate.compareTo(BigDecimal.ZERO) == 0) {
				differenceRate = BigDecimal.ZERO;
			}
		}
		
		return differenceRate;
	}
	
	public void setDifferenceRate(BigDecimal differenceRate) {
		this.differenceRate = differenceRate;
	}
	
	public MaterialPriceCheckResult getParent() {
		return parent;
	}
	
	public void setParent(MaterialPriceCheckResult parent) {
		this.parent = parent;
	}
	
	public String getParentId() {
		return parentId;
	}
	
	public void setParentId(String parentId) {
		this.parentId = parentId;
	}
	
	public BigDecimal getPrice() {
		return price;
	}
	
	public void setPrice(BigDecimal price) {
		this.price = price;
	}
	
	public BigDecimal getComparePrice() {
		return comparePrice;
	}
	
	public void setComparePrice(BigDecimal comparePrice) {
		this.comparePrice = comparePrice;
	}
	
	public List<MaterialPriceCheckResult> getChildList() {
		return childList;
	}
	
	public void setChildList(List<MaterialPriceCheckResult> childList) {
		this.childList = childList;
	}
	
	public void addChild(MaterialPriceCheckResult child) {
		childList.add(child);
	}
}
