package com.gx.obe.business.vo;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.util.utils.BigDecimalUtils;

/** 
 * @Description: 清单
 * @author chenxw 
 */
/**
 * @preserve private
 */
public class QdParameter {
	
	private Logger LOG = Logger.getLogger(QdParameter.class);
	
	private String id;
	
	private String billCode;//清单编码（必填）
	private String billName;//清单名称（必填）
	private BigDecimal quantity;//工程量（必填）
	private String unit;//单位（必填）
	
	private SubTotalParameter subTotalParameter;
	
	private Map<String, SupplierQdParameter> supplierQdParameterMap = new HashMap<String, SupplierQdParameter>();//供应商清单列表
	
	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * @Description: 获取清单编码
	 * @author chenxw
	 * @return
	 */
	public String getBillCode() {
		return billCode;
	}
	
	/**
	 * @Description: 设置清单编码
	 * @author chenxw
	 * @param billCode
	 */
	public void setBillCode(String billCode) {
		this.billCode = billCode;
	}
	
	/**
	 * @Description: 获取清单名称
	 * @author chenxw
	 * @return
	 */
	public String getBillName() {
		return billName;
	}
	
	/**
	 * @Description: 设置清单名称
	 * @author chenxw
	 * @param billName
	 */
	public void setBillName(String billName) {
		this.billName = billName;
	}
	
	/**
	 * @Description: 获取工程量
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getQuantity() {
		return quantity;
	}
	
	/**
	 * @Description: 设置工程量
	 * @author chenxw
	 * @param quantity
	 */
	public void setQuantity(BigDecimal quantity) {
		this.quantity = quantity;
	}
	
	/**
	 * @Description: 获取单位
	 * @author chenxw
	 * @return
	 */
	public String getUnit() {
		return unit;
	}
	
	/**
	 * @Description: 设置单位
	 * @author chenxw
	 * @param unit
	 */
	public void setUnit(String unit) {
		this.unit = unit;
	}
	
	/**
	 * @Description: 获取单价平均值
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnitPriceAverageValue() {
		List<SupplierQdParameter> list = getList(unitPriceComparator);
		BigDecimal subUnitPrice = BigDecimal.ZERO;
		for (SupplierQdParameter supplierQdParameter : list) {
			subUnitPrice = subUnitPrice.add(supplierQdParameter.getUnitPrice());
		}
		return BigDecimalUtils.div(subUnitPrice, BigDecimal.valueOf(list.size()));
	}
	
	/**
	 * @Description: 获取单价控制线
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnitPriceControlLine() {
		return getUnitPriceAverageValue().multiply(subTotalParameter.getControlLineRate());
	}
	
	/**
	 * @Description: 获取合价平均值
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getCombinedPriceAverageValue() {
		List<SupplierQdParameter> list = getList(combinedPriceComparator);
		BigDecimal subCombinedPrice = BigDecimal.ZERO;
		for (SupplierQdParameter supplierQdParameter : list) {
			subCombinedPrice = subCombinedPrice.add(supplierQdParameter.getCombinedPrice());
		}
		return BigDecimalUtils.div(subCombinedPrice, BigDecimal.valueOf(list.size()));
	}
	
	/**
	 * @Description: 获取合价控制线
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getCombinedPriceControlLine() {
		return getCombinedPriceAverageValue().multiply(subTotalParameter.getControlLineRate());
	}
	
	/**
	 * @Description: 获取权重
	 * @author chenxw
	 * @param subCombinedPriceAverageValue
	 * @return
	 */
	public BigDecimal getWeight() {
		if (subTotalParameter.getSubCombinedPriceAverageValue().compareTo(BigDecimal.ZERO) == 0) {
			return BigDecimal.ZERO;
		}
		return BigDecimalUtils.div(getCombinedPriceAverageValue(), subTotalParameter.getSubCombinedPriceAverageValue());
	}
	
	/**
	 * @Description: 获取权重百分比形式
	 * @author chenxw
	 * @return
	 */
	public String getWeightString() {
		return BigDecimalUtils.round(getWeight().multiply(BigDecimal.valueOf(100)), 2) + "%";
	}
	
	/**
	 * @Description: 添加供应商清单表
	 * @author chenxw
	 * @param supplierQdParameter
	 */
	public void addSupplierQdParameter(SupplierQdParameter supplierQdParameter) {
		supplierQdParameter.setQdParameter(this);
		if (supplierQdParameterMap.containsKey(supplierQdParameter.getSupplierId())) {
			LOG.error("billCode = " + billCode + ", supplierId = " + supplierQdParameter.getSupplierId());
		}
		supplierQdParameterMap.put(supplierQdParameter.getSupplierId(), supplierQdParameter);
	}
	
	/**
	 * @Description: 获取供应商清单列表
	 * @author chenxw
	 * @return
	 */
	public SupplierQdParameter getSupplierQdParameter(String supplierId) {
		return supplierQdParameterMap.get(supplierId);
	}
	
	/**
	 * @Description: 获取供应商清单列表
	 * @author chenxw
	 * @return
	 */
	public List<SupplierQdParameter> getSupplierQdParameterList() {
		return new ArrayList<SupplierQdParameter>(supplierQdParameterMap.values());
	}
	
	/**
	 * @Description: 获取供应商清单列表
	 * @author chenxw
	 * @return
	 */
	public Map<String, SupplierQdParameter> getSupplierQdParameterMap() {
		return supplierQdParameterMap;
	}
	
	public void setSubTotalParameter(SubTotalParameter subTotalParameter) {
		this.subTotalParameter = subTotalParameter;
	}
	
	/**
	 * @Description: 获取基准价
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getBasePrice() {
		if (TenderProjectEnum.EXTRACT_COEFFICIENT.equals(subTotalParameter.getEvalMethod())) {
			return getCombinedPriceAverageValue().multiply(subTotalParameter.getBenchmarkPriceRatio());
		} else if (TenderProjectEnum.SUB_LOW_PRICE.equals(subTotalParameter.getEvalMethod())) {
			TreeSet<SupplierQdParameter> supplierQdParameterSet = getFiltersupplierQdParameterList();
			if (supplierQdParameterSet.size() > 1) {
				return supplierQdParameterSet.higher(supplierQdParameterSet.first()).getCombinedPrice();
			}
		}
		return null;
	}
	
	/**
	 * @Description: 获取不低于成本清单列表
	 * @author chenxw
	 * @return
	 */
	private TreeSet<SupplierQdParameter> getFiltersupplierQdParameterList() {
		TreeSet<SupplierQdParameter> supplierQdParameterSet = new TreeSet<SupplierQdParameter>(combinedPriceComparator);
		for (SupplierQdParameter supplierTotalProjectParameter : getSupplierQdParameterList()) {
			if (supplierTotalProjectParameter.isBelowCost()) {
				continue;
			}
			supplierQdParameterSet.add(supplierTotalProjectParameter);
		}
		return supplierQdParameterSet;
	}
	
	/**
	 * @Description: 获取基准分
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getBenchmarkScore() {
		return subTotalParameter.getBenchmarkScore().multiply(getWeight());
	}
	
	/**
	 * @Description: 获取高于基准价扣分系数
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getExceedBenchmarkPriceDeductScoreRatio() {
		return subTotalParameter.getExceedBenchmarkPriceDeductScoreRatio();
	}
	
	/**
	 * @Description: 获取低于基准价扣分系数
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnderBenchmarkPriceDeductScoreRatio() {
		return subTotalParameter.getUnderBenchmarkPriceDeductScoreRatio();
	}
	
	/**
	 * @Description: 获取筛选后的供应商列表
	 * @author chenxw
	 * @param comparator
	 * @return
	 */
	private List<SupplierQdParameter> getList(Comparator<? super SupplierQdParameter> comparator) {
		int supplierCount = supplierQdParameterMap.size();
		int removeCount = 0;
		if (supplierCount > 20) {
			removeCount = 3;
		} else if (supplierCount > 13) {
			removeCount = 2;
		} else if (supplierCount > 7) {
			removeCount = 1;
		}
		List<SupplierQdParameter> list = new ArrayList<SupplierQdParameter>(supplierQdParameterMap.values());
		Collections.sort(list, comparator);
		for (int i = 0; i < removeCount; i++) {
			list.remove(0);
			list.remove(list.size() - 1);
		}
		return list;
	}
	
	/**
	 * 单价排列
	 */
	Comparator<SupplierQdParameter> unitPriceComparator = new Comparator<SupplierQdParameter>() {
		public int compare(SupplierQdParameter o1, SupplierQdParameter o2) {
			return o1.getUnitPrice().compareTo(o2.getUnitPrice());
		}
	};
	
	/**
	 * 合价排列
	 */
	Comparator<SupplierQdParameter> combinedPriceComparator = new Comparator<SupplierQdParameter>() {
		public int compare(SupplierQdParameter o1, SupplierQdParameter o2) {
			return o1.getCombinedPrice().compareTo(o2.getCombinedPrice());
		}
	};
	
}
