package com.gx.obe.business.vo;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.gx.obe.util.utils.ObjectUtils;

/** 
 * @Description: 所有分部分项
 * @author chenxw 
 */
/**
 * @preserve private
 */
public class SubTotalParameter {
	
	public static final String BILL_ITEMIZE = "itemize";
	public static final String BILL_MEASURE = "measure";
	
	private String id;
	
	private String evaluationType;
	
	private String evalMethod;//评标办法（统一必填）
	private BigDecimal controlLineRate;//控制线系数（统一必填）
	
	private BigDecimal benchmarkScore;//基准分（必填）
	
	private BigDecimal benchmarkPriceRatio;//基准价系数（抽取系数法必填）
	
	private BigDecimal exceedBenchmarkPriceDeductScoreRatio;//高于基准价系数（综合评价法必填）
	private BigDecimal underBenchmarkPriceDeductScoreRatio;//低于基准价系数（综合评价法必填）
	
	private Map<String, TotalParameter> totalParameterMap = new HashMap<String, TotalParameter>();//分部分项列表
	
	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * @Description: 获取分部分项列表
	 * @author chenxw
	 * @return
	 */
	public Map<String, TotalParameter> getTotalParameterMap() {
		return totalParameterMap;
	}
	
	/**
	 * @Description: 获取分部分项
	 * @author chenxw
	 * @param key
	 * @return
	 */
	public TotalParameter get(String key) {
		return totalParameterMap.get(key);
	}
	
	/**
	 * @Description: 是否含有此分部分项
	 * @author chenxw
	 * @param key
	 * @return
	 */
	public boolean containsKey(String key) {
		return totalParameterMap.containsKey(key);
	}
	
	/**
	 * @Description: 添加分部分项
	 * @author chenxw
	 * @param totalParameter
	 */
	public void addTotalParameter(TotalParameter totalParameter) {
		totalParameter.setSubFbfxParameter(this);
		this.totalParameterMap.put(totalParameter.getProjectId(), totalParameter);
	}
	
	/**
	 * @Description: 获取合价平均值之和
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getSubCombinedPriceAverageValue() {
		BigDecimal subCombinedPriceAverageValue = BigDecimal.ZERO;
		for (String projectId : totalParameterMap.keySet()) {
			Map<String, QdParameter> qdParameterMap = totalParameterMap.get(projectId).getQdParameterMap();
			if (null != qdParameterMap) {
				for (String qdParameterId : qdParameterMap.keySet()) {
					subCombinedPriceAverageValue = subCombinedPriceAverageValue.add(qdParameterMap.get(qdParameterId).getCombinedPriceAverageValue());
				}
			}
		}
		return subCombinedPriceAverageValue;
	}
	
	/**
	 * @Description: 设置基准价系数
	 * @author chenxw
	 * @param benchmarkPriceRatio
	 */
	public void setBenchmarkPriceRatio(BigDecimal benchmarkPriceRatio) {
		this.benchmarkPriceRatio = benchmarkPriceRatio;
	}
	
	/**
	 * @Description: 获取基准价系数
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getBenchmarkPriceRatio() {
		return benchmarkPriceRatio;
	}
	
	/**
	 * @Description: 设置高于基准价扣分系数
	 * @author chenxw
	 * @param exceedBenchmarkPriceDeductScoreRatio
	 */
	public void setExceedBenchmarkPriceDeductScoreRatio(BigDecimal exceedBenchmarkPriceDeductScoreRatio) {
		this.exceedBenchmarkPriceDeductScoreRatio = exceedBenchmarkPriceDeductScoreRatio;
	}
	
	/**
	 * @Description: 获取高于基准价扣分系数
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getExceedBenchmarkPriceDeductScoreRatio() {
		return exceedBenchmarkPriceDeductScoreRatio;
	}
	
	/**
	 * @Description: 设置低于基准价扣分系数
	 * @author chenxw
	 * @param underBenchmarkPriceDeductScoreRatio
	 */
	public void setUnderBenchmarkPriceDeductScoreRatio(BigDecimal underBenchmarkPriceDeductScoreRatio) {
		this.underBenchmarkPriceDeductScoreRatio = underBenchmarkPriceDeductScoreRatio;
	}
	
	/**
	 * @Description: 获取低于基准价扣分系数
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnderBenchmarkPriceDeductScoreRatio() {
		return underBenchmarkPriceDeductScoreRatio;
	}
	
	/**
	 * @Description: 获取基准分（分部分项分值）
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getBenchmarkScore() {
		return benchmarkScore;
	}
	
	/**
	 * @Description: 设置基准分（分部分项分值）
	 * @author chenxw
	 * @param benchmarkScore
	 */
	public void setBenchmarkScore(BigDecimal benchmarkScore) {
		this.benchmarkScore = benchmarkScore;
	}
	
	/**
	 * @Description: 获取评标办法
	 * @author chenxw
	 * @return
	 */
	public String getEvalMethod() {
		return evalMethod;
	}
	
	public void setEvalMethod(String evalMethod) {
		this.evalMethod = evalMethod;
	}
	
	/**
	 * @Description: 加载分部分项列表
	 * @author chenxw
	 * @param supplierQdList
	 * @param controlLineRate
	 * @param subTotalParameter
	 * @return
	 */
	public void initTotalParameterList(List<Map<String, Object>> supplierQdList) {
		if (null == supplierQdList) {
			return;
		}
		for (Map<String, Object> supplierQd : supplierQdList) {
			String projectId = ObjectUtils.getObjString(supplierQd.get("PROJECT_ID"));
			if (containsKey(projectId)) {
				TotalParameter totalParameter = get(projectId);
				String billId = ObjectUtils.getObjString(supplierQd.get("QD_ID"));
				if (totalParameter.containsKey(billId)) {
					totalParameter.get(billId).addSupplierQdParameter(getSupplierQdParameter(supplierQd));
				} else {
					totalParameter.addQdParameter(getQdParameter(supplierQd));
				}
			} else {
				TotalParameter totalParameter = new TotalParameter();
				totalParameter.setProjectId(projectId);
				addTotalParameter(totalParameter);
				totalParameter.addQdParameter(getQdParameter(supplierQd));
			}
		}
	}
	
	/**
	 * @Description: 获取清单
	 * @author chenxw
	 * @param supplierQd
	 * @return
	 */
	private QdParameter getQdParameter(Map<String, Object> supplierQd) {
		QdParameter qdParameter = new QdParameter();
		qdParameter.setId(ObjectUtils.getObjString(supplierQd.get("QD_ID")));
		qdParameter.setBillCode(ObjectUtils.getObjString(supplierQd.get("QD_CODE")));
		qdParameter.setBillName(ObjectUtils.getObjString(supplierQd.get("QD_NAME")));
		qdParameter.setQuantity(ObjectUtils.getObjBigDecimal(supplierQd.get("QUANTITY"), 2));
		qdParameter.setUnit(ObjectUtils.getObjString(supplierQd.get("UNIT")));
		qdParameter.addSupplierQdParameter(getSupplierQdParameter(supplierQd));
		return qdParameter;
	}
	
	/**
	 * @Description: 获取供应商清单
	 * @author chenxw
	 * @param supplierQd
	 * @return
	 */
	private SupplierQdParameter getSupplierQdParameter(Map<String, Object> supplierQd) {
		SupplierQdParameter supplierQdParameter = new SupplierQdParameter();
		supplierQdParameter.setId(ObjectUtils.getObjString(supplierQd.get("SUPPLIER_QD_ID")));
		supplierQdParameter.setSupplierId(ObjectUtils.getObjString(supplierQd.get("SUPPLIER_ID")));
		supplierQdParameter.setUnitPrice(ObjectUtils.getObjBigDecimal(supplierQd.get("UNIT_PRICE"), 2));
		supplierQdParameter.setCombinedPrice(ObjectUtils.getObjBigDecimal(supplierQd.get("COMBINED_PRICE"), 2));
		return supplierQdParameter;
	}
	
	public void setControlLineRate(BigDecimal controlLineRate) {
		this.controlLineRate = controlLineRate;
	}
	
	public BigDecimal getControlLineRate() {
		return controlLineRate;
	}
	
	public void setEvaluationType(String evaluationType) {
		this.evaluationType = evaluationType;
	}
	
	public String getEvaluationType() {
		return evaluationType;
	}
	
}
