package com.gx.obe.business.vo;

import java.math.BigDecimal;

import com.gx.obe.util.utils.BigDecimalUtils;

/** 
 * @Description: 供应商清单
 * @author chenxw 
 */
/**
 * @preserve private
 */
public class SupplierQdParameter {
	
	private String id;
	
	private String supplierId;//供应商Id
	
	private BigDecimal unitPrice;//单价（必填）
	private BigDecimal combinedPrice;//合价（必填）
	
	private QdParameter qdParameter;//清单
	
	private boolean belowCost = false;//是否低于成本（必填）
	
	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * @Description: 获取供应商Id
	 * @author chenxw
	 * @return
	 */
	public String getSupplierId() {
		return supplierId;
	}
	
	/**
	 * @Description: 设置供应商Id
	 * @author chenxw
	 * @param supplierId
	 */
	public void setSupplierId(String supplierId) {
		this.supplierId = supplierId;
	}
	
	/**
	 * @Description: 获取单价
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnitPrice() {
		if (null == unitPrice) {
			unitPrice = BigDecimal.ZERO;
		}
		return unitPrice;
	}
	
	/**
	 * @Description: 设置单价
	 * @author chenxw
	 * @param unitPrice
	 */
	public void setUnitPrice(BigDecimal unitPrice) {
		this.unitPrice = unitPrice;
	}
	
	/**
	 * @Description: 获取合价
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getCombinedPrice() {
		if (null == combinedPrice) {
			combinedPrice = BigDecimal.ZERO;
		}
		return combinedPrice;
	}
	
	/**
	 * @Description: 设置合价
	 * @author chenxw
	 * @param combinedPrice
	 */
	public void setCombinedPrice(BigDecimal combinedPrice) {
		this.combinedPrice = combinedPrice;
	}
	
	/**
	 * @Description: 获取单价平均值
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnitPriceAverageValue() {
		return qdParameter.getUnitPriceAverageValue();
	}
	
	/**
	 * @Description: 获取合价平均值
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getCombinedPriceAverageValue() {
		return qdParameter.getCombinedPriceAverageValue();
	}
	
	/**
	 * @Description: 获取单价控制线
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnitPriceControlLine() {
		return qdParameter.getUnitPriceControlLine();
	}
	
	/**
	 * @Description: 获取合价控制线
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getCombinedPriceControlLine() {
		return qdParameter.getCombinedPriceControlLine();
	}
	
	/**
	 * @Description: 获取报价差额
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getOfferDeviation() {
		return getCombinedPrice().subtract(getCombinedPriceControlLine());
	}
	
	/**
	 * @Description: 获取偏差率
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getDeviationRate() {
		BigDecimal combinedPriceControlLine = getCombinedPriceControlLine();
		if (combinedPriceControlLine.compareTo(BigDecimal.ZERO) == 0) {
			return BigDecimal.ZERO;
		}
		return BigDecimalUtils.div(getOfferDeviation(), getCombinedPriceControlLine());
	}
	
	/**
	 * @Description: 获取偏差率百分比形式
	 * @author chenxw
	 * @return
	 */
	public String getDeviationPercentageString() {
		return BigDecimalUtils.round(getDeviationRate().multiply(BigDecimal.valueOf(100)), 2) + "%";
	}
	
	/**
	 * @Description: 获取偏差金额
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getDeviationPrice() {
		return getCombinedPrice().subtract(qdParameter.getBasePrice());
	}
	
	/**
	 * @Description: 获取百分制偏差率
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getDeviationCentesimalRatio() {
		if (null == qdParameter.getBasePrice() || qdParameter.getBasePrice().compareTo(BigDecimal.ZERO) == 0) {
			return BigDecimal.ZERO;
		}
		return BigDecimalUtils.div(getDeviationPrice().multiply(BigDecimal.valueOf(100)), qdParameter.getBasePrice());
	}
	
	/**
	 * @Description: 获取得分
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getScore() {
		if (isBelowCost() || getCombinedPrice().compareTo(BigDecimal.ZERO) == 0) {
			return BigDecimal.ZERO;
		}
		BigDecimal score = qdParameter.getBenchmarkScore();
		BigDecimal deviationScore = qdParameter.getBenchmarkScore().multiply(getDeviationCentesimalRatio());
		if (getDeviationCentesimalRatio().compareTo(BigDecimal.ZERO) > 0) {
			score = score.subtract(qdParameter.getExceedBenchmarkPriceDeductScoreRatio().multiply(deviationScore));
		}
		if (getDeviationCentesimalRatio().compareTo(BigDecimal.ZERO) < 0) {
			score = score.add(qdParameter.getUnderBenchmarkPriceDeductScoreRatio().multiply(deviationScore));
		}
		if (score.compareTo(BigDecimal.ZERO) < 0) {
			score = BigDecimal.ZERO;
		}
		return score;
	}
	
	/**
	 * @Description: 设置清单
	 * @author chenxw
	 * @param qdParameter
	 */
	public void setQdParameter(QdParameter qdParameter) {
		this.qdParameter = qdParameter;
	}
	
	/**
	 * @Description: 是不是不平衡报价
	 * @author chenxw
	 * @return
	 */
	public boolean isImbalancePrice() {
		return getCombinedPriceControlLine().multiply(BigDecimal.valueOf(2)).compareTo(getCombinedPrice()) < 0;
	}
	
	/**
	 * @Description: 是否低于成本
	 * @author chenxw
	 * @return
	 */
	public boolean isBelowCost() {
		return belowCost;
	}
	
	/**
	 * @Description: 设置是否低于成本
	 * @author chenxw
	 * @param belowCost
	 */
	public void setBelowCost(boolean belowCost) {
		this.belowCost = belowCost;
	}
	
}
