package com.gx.obe.business.vo;

import java.math.BigDecimal;

import com.gx.obe.util.utils.BigDecimalUtils;

/**
 * @preserve private
 */
public class SupplierTotalProjectParameter {
	
	private String id;
	
	private String supplierId;
	
	private BigDecimal totalPrice;//总价
	
	private TotalProjectParameter totalProjectParameter;
	
	private boolean belowCost = false;//是否低于成本
	
	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * @Description: 设置投标人主键
	 * @author chenxw
	 * @return
	 */
	public String getSupplierId() {
		return supplierId;
	}
	
	/**
	 * @Description: 获取投标人主键
	 * @author chenxw
	 * @param supplierId
	 */
	public void setSupplierId(String supplierId) {
		this.supplierId = supplierId;
	}
	
	/**
	 * @Description: 获取总价
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getTotalPrice() {
		return totalPrice;
	}
	
	/**
	 * @Description: 设置总价
	 * @author chenxw
	 * @param totalPrice
	 */
	public void setTotalPrice(BigDecimal totalPrice) {
		this.totalPrice = totalPrice;
	}
	
	/**
	 * @Description: 获取总价平均值
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getTotalPriceAverageValue() {
		return totalProjectParameter.getTotalPriceAverageValue();
	}
	
	/**
	 * @Description: 获取总价控制线
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getTotalPriceControlLine() {
		return totalProjectParameter.getTotalPriceControlLine();
	}
	
	/**
	 * @Description: 获取报价差额
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getOfferDeviation() {
		return getTotalPrice().subtract(getTotalPriceControlLine());
	}
	
	/**
	 * @Description: 获取偏差率
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getDeviationPercentage() {
		return BigDecimalUtils.div(getOfferDeviation(), getTotalPriceControlLine());
	}
	
	/**
	 * @Description: 获取偏差率百分比形式
	 * @author chenxw
	 * @return
	 */
	public String getDeviationPercentageString() {
		return BigDecimalUtils.round(getDeviationPercentage().multiply(BigDecimal.valueOf(100)), 2) + "%";
	}
	
	/**
	 * @Description: 设置总工程
	 * @author chenxw
	 * @param totalProjectParameter
	 */
	public void setTotalProjectParameter(TotalProjectParameter totalProjectParameter) {
		this.totalProjectParameter = totalProjectParameter;
	}
	
	/**
	 * @Description: 获取偏差金额
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getDeviationPrice() {
		return getTotalPrice().subtract(totalProjectParameter.getBenchmarkPrice());
	}
	
	/**
	 * @Description: 获取百分制偏差率
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getDeviationCentesimalRatio() {
		return BigDecimalUtils.div(getDeviationPrice().multiply(BigDecimal.valueOf(100)), totalProjectParameter.getBenchmarkPrice());
	}
	
	/**
	 * @Description: 获取得分
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getScore() {
		if (getTotalPrice().compareTo(BigDecimal.ZERO) == 0) {
			return BigDecimal.ZERO;
		}
		BigDecimal score = totalProjectParameter.getBenchmarkScore();
		if (getDeviationCentesimalRatio().compareTo(BigDecimal.ZERO) > 0) {
			score = score.subtract(totalProjectParameter.getExceedBenchmarkPriceDeductScore().multiply(getDeviationCentesimalRatio()));
		}
		if (getDeviationCentesimalRatio().compareTo(BigDecimal.ZERO) < 0) {
			score = score.add(totalProjectParameter.getUnderBenchmarkPriceDeductScore().multiply(getDeviationCentesimalRatio()));
		}
		if (score.compareTo(BigDecimal.ZERO) < 0) {
			score = BigDecimal.ZERO;
		}
		return score;
	}
	
	public boolean isBelowCost() {
		return belowCost;
	}
	
	public void setBelowCost(boolean belowCost) {
		this.belowCost = belowCost;
	}
	
}
