package com.gx.obe.business.vo;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.ObjectUtils;

/**
 * @preserve private
 */
public class TotalProjectParameter {
	
	private String id;
	
	private String evalMethod;//评标办法（统一必填）
	private BigDecimal controlLineRate;//控制线系数（统一必填）
	
	private BigDecimal benchmarkPriceRatio;//基准价系数（抽取系数法统一必填）
	
	private BigDecimal benchmarkScore;//基准分（必填）
	
	private BigDecimal exceedBenchmarkPriceDeductScore;//高于基准价系数（综合评价法必填）
	private BigDecimal underBenchmarkPriceDeductScore;//低于基准价系数（综合评价法必填）
	
	private List<SupplierTotalProjectParameter> supplierTotalProjectParameterList = new ArrayList<SupplierTotalProjectParameter>();
	
	public String getId() {
		return id;
	}
	
	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * @Description: 设置控制线系数
	 * @author chenxw
	 * @param controlLineRate
	 */
	public void setControlLineRate(BigDecimal controlLineRate) {
		this.controlLineRate = controlLineRate;
	}
	
	/**
	 * @Description: 获取总价平均值
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getTotalPriceAverageValue() {
		List<SupplierTotalProjectParameter> list = getList(totalPriceComparator);
		BigDecimal subTotalPrice = BigDecimal.ZERO;
		if (null != list) {
			for (SupplierTotalProjectParameter supplierTotalProjectParameter : list) {
				subTotalPrice = BigDecimalUtils.add(subTotalPrice, supplierTotalProjectParameter.getTotalPrice());
			}
			if (list.size() > 1) {
				return BigDecimalUtils.div(subTotalPrice, BigDecimal.valueOf(list.size()), 2);
			}
		}
		return subTotalPrice;
	}
	
	/**
	 * @Description: 获取总价控制线
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getTotalPriceControlLine() {
		return getTotalPriceAverageValue().multiply(controlLineRate);
	}
	
	/**
	 * @Description: 获取投标人总工程列表
	 * @author chenxw
	 * @return
	 */
	public List<SupplierTotalProjectParameter> getSupplierTotalProjectParameterList() {
		return supplierTotalProjectParameterList;
	}
	
	/**
	 * @Description: 添加投标人总工程
	 * @author chenxw
	 * @param supplierTotalProjectParameter
	 */
	public void addSupplierTotalProjectParameter(SupplierTotalProjectParameter supplierTotalProjectParameter) {
		supplierTotalProjectParameter.setTotalProjectParameter(this);
		supplierTotalProjectParameterList.add(supplierTotalProjectParameter);
	}
	
	/**
	 * @Description: 获取基准价
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getBenchmarkPrice() {
		if (TenderProjectEnum.EXTRACT_COEFFICIENT.equals(evalMethod)) {
			return BigDecimalUtils.mul(getTotalPriceAverageValue(), benchmarkPriceRatio);
		} else if (TenderProjectEnum.SUB_LOW_PRICE.equals(evalMethod)) {
			List<SupplierTotalProjectParameter> filterSupplierTotalProjectParameterList = getFilterSupplierTotalProjectParameterList();
			Collections.sort(filterSupplierTotalProjectParameterList, totalPriceComparator);
			if (filterSupplierTotalProjectParameterList.size() > 1) {
				return filterSupplierTotalProjectParameterList.get(1).getTotalPrice();
			}
		}
		return BigDecimal.ZERO;
	}
	
	/**
	 * @Description: 设置基准价系数
	 * @author chenxw
	 * @param benchmarkPriceRatio
	 */
	public void setBenchmarkPriceRatio(BigDecimal benchmarkPriceRatio) {
		this.benchmarkPriceRatio = benchmarkPriceRatio;
	}
	
	/**
	 * @Description: 设置高于基准价扣分
	 * @author chenxw
	 * @param exceedBenchmarkPriceDeductScore
	 */
	public void setExceedBenchmarkPriceDeductScore(BigDecimal exceedBenchmarkPriceDeductScore) {
		this.exceedBenchmarkPriceDeductScore = exceedBenchmarkPriceDeductScore;
	}
	
	/**
	 * @Description: 获取高于基准价扣分
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getExceedBenchmarkPriceDeductScore() {
		return exceedBenchmarkPriceDeductScore;
	}
	
	/**
	 * @Description: 设置低于基准价扣分
	 * @author chenxw
	 * @param underBenchmarkPriceDeductScore
	 */
	public void setUnderBenchmarkPriceDeductScore(BigDecimal underBenchmarkPriceDeductScore) {
		this.underBenchmarkPriceDeductScore = underBenchmarkPriceDeductScore;
	}
	
	/**
	 * @Description: 获取低于基准价扣分
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getUnderBenchmarkPriceDeductScore() {
		return underBenchmarkPriceDeductScore;
	}
	
	/**
	 * @Description: 获取基准分（总价分值）
	 * @author chenxw
	 * @return
	 */
	public BigDecimal getBenchmarkScore() {
		return benchmarkScore;
	}
	
	/**
	 * @Description: 设置基准分（总价分值）
	 * @author chenxw
	 * @param benchmarkScore
	 */
	public void setBenchmarkScore(BigDecimal benchmarkScore) {
		this.benchmarkScore = benchmarkScore;
	}
	
	/**
	 * @Description: 获取不低于成本的投标人清单列表
	 * @author chenxw
	 * @return
	 */
	private List<SupplierTotalProjectParameter> getFilterSupplierTotalProjectParameterList() {
		List<SupplierTotalProjectParameter> filterSupplierTotalProjectParameterList = new ArrayList<SupplierTotalProjectParameter>();
		for (SupplierTotalProjectParameter supplierTotalProjectParameter : supplierTotalProjectParameterList) {
			if (supplierTotalProjectParameter.isBelowCost()) {
				continue;
			}
			filterSupplierTotalProjectParameterList.add(supplierTotalProjectParameter);
		}
		return filterSupplierTotalProjectParameterList;
	}
	
	/**
	 * @Description: 获取筛选后的供应商列表
	 * @author chenxw
	 * @param comparator
	 * @return
	 */
	private List<SupplierTotalProjectParameter> getList(Comparator<? super SupplierTotalProjectParameter> comparator) {
		int supplierCount = supplierTotalProjectParameterList.size();
		int removeCount = 0;
		if (supplierCount > 20) {
			removeCount = 3;
		} else if (supplierCount > 13) {
			removeCount = 2;
		} else if (supplierCount > 7) {
			removeCount = 1;
		}
		List<SupplierTotalProjectParameter> list = new ArrayList<SupplierTotalProjectParameter>(supplierTotalProjectParameterList);
		Collections.sort(list, comparator);
		for (int i = 0; i < removeCount; i++) {
			list.remove(0);
			list.remove(list.size() - 1);
		}
		return list;
	}
	
	public void setEvalMethod(String evalMethod) {
		this.evalMethod = evalMethod;
	}
	
	public String getEvalMethod() {
		return evalMethod;
	}
	
	public void initTotalProjectParameter(List<Map<String, Object>> supplierTotalProjectList) {
		if (null != supplierTotalProjectList) {
			for (Map<String, Object> supplierTotalProject : supplierTotalProjectList) {
				addSupplierTotalProjectParameter(getSupplierTotalProjectParameter(supplierTotalProject));
			}
		}
	}
	
	private SupplierTotalProjectParameter getSupplierTotalProjectParameter(Map<String, Object> supplierTotalProject) {
		SupplierTotalProjectParameter supplierTotalProjectParameter = new SupplierTotalProjectParameter();
		supplierTotalProjectParameter.setSupplierId(ObjectUtils.getObjString(supplierTotalProject.get("SUPPLIER_ID")));
		supplierTotalProjectParameter.setTotalPrice(ObjectUtils.getObjBigDecimal(supplierTotalProject.get("TOTAL_PRICE")));
		return supplierTotalProjectParameter;
	}
	
	/**
	 * 总价排列
	 */
	Comparator<SupplierTotalProjectParameter> totalPriceComparator = new Comparator<SupplierTotalProjectParameter>() {
		public int compare(SupplierTotalProjectParameter o1, SupplierTotalProjectParameter o2) {
			return o1.getTotalPrice().compareTo(o2.getTotalPrice());
		}
	};
	
}
