package com.gx.obe.web.utils;

import java.util.HashMap;
import java.util.Map;

import com.gx.obe.business.listener.HttpStatusMonitor;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.HttpResponse;
import com.gx.obe.http.HttpStatus;
import com.gx.obe.http.HttpUtil; 
/** 
 * @Description: 网络请求
 * @author mazc 
 */
public class NetworkRequest {
	
	
	public static String getNoHeader(String strWebServerUrl, Map<String, Object> paramMap){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).form(paramMap).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	
	public static String postNoHeader(String strWebServerUrl, Map<String, Object> paramMap){
		return postNoHeader(strWebServerUrl,paramMap,false);
	}
	
	public static String postNoHeader(String strWebServerUrl, Map<String, Object> paramMap,boolean isRest){
		HttpResponse httpResponse = HttpUtil.createPost(strWebServerUrl).form(paramMap).setRest(isRest).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	
	

	/**
	 * @Description: Get请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param paramMap
	 * @return 
	 */
	public static String get(String strWebServerUrl, Map<String, Object> paramMap){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).setHeader(getHeaderMap(), true).form(paramMap).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	
	public static String get(String strWebServerUrl, Map<String, Object> paramMap ,int timeout){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(timeout).form(paramMap).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	public static String get(String strWebServerUrl){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).setHeader(getHeaderMap(), true).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	public static HttpResponse getResponse(String strWebServerUrl){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).setHeader(getHeaderMap(), true).execute();
		stateProces(httpResponse);
		return httpResponse;
	} 
	
	/**
	 * @Description:  Get请求返回HttpResponse
	 * @author mazc
	 * @param strWebServerUrl
	 * @param paramMap
	 * @return HttpResponse
	 */
	public static HttpResponse getHttpResponse(String strWebServerUrl, Map<String, Object> paramMap){
		HttpResponse httpResponse = HttpUtil.createGet(strWebServerUrl).setHeader(getHeaderMap(), true).form(paramMap).execute();
		stateProces(httpResponse);
		return httpResponse;
	} 
	
	
	/**
	 * @Description: post请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param body
	 * @return String
	 */
	public static String post(String strWebServerUrl, Map<String, Object> paramMap){
		HttpResponse httpResponse = HttpUtil.createPost(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(-1).form(paramMap).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	/**
	 * @Description: post请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param body
	 * @return String
	 */
	public static String post(String strWebServerUrl, String body){
		HttpResponse httpResponse = HttpUtil.createPost(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(-1).body(body).execute();
		stateProces(httpResponse);
		return httpResponse.body();
	} 
	
	/**
	 * @Description: post请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param body
	 * @return HttpResponse
	 */
	public static HttpResponse postHttpResponse(String strWebServerUrl, String body){
		HttpResponse httpResponse = HttpUtil.createPost(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(-1).body(body).execute();
		stateProces(httpResponse);
		return httpResponse;
	} 
	
	/**
	 * @Description: post请求
	 * @author mazc
	 * @param strWebServerUrl
	 * @param body
	 * @return HttpResponse
	 */
	public static HttpResponse postHttpResponse(String strWebServerUrl, Map<String, Object> paramMap){
		HttpResponse httpResponse = HttpUtil.createPost(strWebServerUrl).setHeader(getHeaderMap(), true).timeout(-1).form(paramMap).execute();
		stateProces(httpResponse);
		return httpResponse;
	} 
	/**
	 * @Description: 统一管理
	 * @author mazc
	 * @return 
	 */
	private static Map<String, String> getHeaderMap(){
		Map<String, String> headerMap = new HashMap<String, String>();
		if(null != Constants.USER && null != Constants.USER.getUserAccount()) {
			headerMap.put("userName", Constants.USER.getUserAccount());
		}
		headerMap.put("ATUOKTHEN", Constants.TOKEN);
		return headerMap;
	}
	
	private static void stateProces(HttpResponse httpResponse) {
		switch (httpResponse.getStatus()) {
    	case HttpStatus.HTTP_UNAUTHORIZED:
    		HttpStatusMonitor.getInstance().HttpStatus(HttpStatus.HTTP_UNAUTHORIZED);
    	default:
    		return;
    	}
		
	}
	
}
