package com.gx.obe.web.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.business.entity.Result;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.http.jsons.JSONNull;
import com.gx.obe.web.utils.NetworkRequest;
import com.gx.obe.web.utils.ResultStatus;

public class ObeHttpUtils {
	
	private static final Logger logger = Logger.getLogger(ObeHttpUtils.class);
	
	public static boolean post(String path, Object obj) {
		String strWebServerUrl = Constants.getServiceUrl(path);
		try {
			return ResultStatus.getReultStatusBoolean(NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(obj)));
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
	}
	
	public static <T> T getOne(String path, Map<String, Object> param, Class<T> tclass) {
		String strWebServerUrl = Constants.getServiceUrl(path);
		try {
			String result = NetworkRequest.get(strWebServerUrl, param);
			if (ResultStatus.getReultStatusOK(result)) {
				return JsonUtil.jsonToBean(result, tclass);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}
	
	public static <T> List<T> getList(String path, Map<String, Object> param, Class<T> tclass) {
		String strWebServerUrl = Constants.getServiceUrl(path);
		try {
			String result = NetworkRequest.get(strWebServerUrl, param);
			if (ResultStatus.getReultStatusOK(result)) {
				return JsonUtil.jsonToList(result, tclass);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return new ArrayList<>();
	}
	public static boolean getBoolean(String path, Map<String, Object> param) {
		String strWebServerUrl = Constants.getServiceUrl(path);
		try {
			return ResultStatus.getReultStatusBoolean(NetworkRequest.get(strWebServerUrl, param));
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
	}
	public static <T> Result<T> getResult(String path, Map<String, Object> param, Class<T> tclass) {
		String strWebServerUrl = Constants.getServiceUrl(path);
		try {
			String result = NetworkRequest.get(strWebServerUrl, param);
			if (ResultStatus.getReultStatusOK(result)) {
				@SuppressWarnings("unchecked")
				Result<T> jsonToBean = JsonUtil.jsonToBean(result, Result.class);
				if (jsonToBean != null) {
					T data = jsonToBean.getData();
					if (data != null && data != JSONNull.NULL) {
						jsonToBean.setData(JsonUtil.jsonToBean(data, tclass));
					}
					return jsonToBean;
				}
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}
	
}
