package com.gx.utils;

import java.util.Date;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.apache.log4j.Logger;

import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.web.entity.log.SysLog;
import com.gx.obe.web.service.log.SysLogService;


public class ActionLogUtils {

//	private static ActionLogService actionLogService = ServiceFactory.getInstance("actionLogService", ActionLogService.class);
//	private static ActionLogDao actionLogDao = ServiceFactory.getInstance("actionLogDao", ActionLogDao.class);
	
	private static SysLogService sysLogService = new SysLogService();
	private static ThreadPoolExecutor logThreadPoolExecutor;
	static{
		logThreadPoolExecutor = new ThreadPoolExecutor(2, 5, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
	}
	private static Logger LOG = Logger.getLogger(ActionLogUtils.class);
	
 
	/**
	 * @param businessId 业务id
	 * @param action  操作事件
	 * @param params 操作内容，说明或参数
	 */
	public static void addServerLog(String businessId ,String action, HttpUrl httpUrl, boolean flag){
		if(null != httpUrl) {
			doLog(businessId ,action, httpUrl.getUrl() + httpUrl.getAttributs(), httpUrl.getHttpResponse().getContent(), httpUrl.getException(), flag ? 1 : -1);
		}else {
			doLog(businessId ,action, null, null, null, flag ? 1 : -1);
		}
	}
	/**
	 * @param businessId 业务id
	 * @param action  操作事件
	 * @param params 操作内容，说明或参数
	 */
	public static void addServerLog(String businessId ,String action,String params){
		doLog(businessId ,action, params, null, null, 6);
	}
	
	/**
	 * @param businessId 业务id
	 * @param action  操作事件
	 * @param params 操作内容，说明或参数
	 */
	public static void addServerLog(String businessId ,String action, Params params){
		doLog(businessId ,action, null != params ? JsonUtil.jacksonObjectToStr(params) : null, null, params.getException(), null != params && !params.isFlag() ? -1 : 6);
	}
	
	/**
	 * @param businessId 业务id
	 * @param action  操作事件
	 * @param params 操作内容，说明或参数
	 * @param returnValue 操作内容，说明或参数
	 */
	public static void addServerLog(String businessId ,String action, String params, String returnValue){
		doLog(businessId, action, params, returnValue, null, 6);
	}
		
	/**
	 * @param businessId 业务id
	 * @param action  操作事件
	 * @param params 操作内容，说明或参数
	 * @param returnValue 操作内容，说明或参数
	 */
	public static void addServerLog(String businessId ,String action, String params, String returnValue, String exception){
		doLog(businessId, action, params, returnValue, exception, 6);
	}
	
	 private static void doLog(final String businessId, final String action, final String params, final String returnValue, String exception, int level) {
		 logThreadPoolExecutor.submit(new Runnable(){
				public void run() {
					SysLog sysLog = new SysLog();
					sysLog.setTitle(action);
					sysLog.setLevel(level);
					sysLog.setParams(params);
					sysLog.setReturnValue(returnValue);
					sysLog.setException(exception);
					sysLog.setCreateUser(Constants.USER.getUserAccount());
					sysLogService.addOperationLog(sysLog);
					LOG.info("["+DateUtils.getTimestampFormat(new Date()) + "]\t[" + Constants.USER.getUserName() + "]\t["+action + "]" + (null != params ? "\t[" + params + "]":"") + "\r\n"+(null != returnValue ? returnValue : ""));
				}
			});
	 }
	
//	private static ActionLog initSystemLog(String businessId, String action, String description){
//		ActionLog actionLog = new ActionLog();
//		actionLog.setId(UuidUtils.getUUID());
//		actionLog.setBusinessId(businessId);
//		if(null != action && action.length() > 100){
//			actionLog.setAction(action.substring(0, 100));
//		}else {
//			actionLog.setAction(action);
//		}
//		if(null != description && description.length() > 500){
//			actionLog.setDescription(description.substring(0, 500));
//		}else {
//			actionLog.setDescription(description);
//		}
//		actionLog.setOperatorId(Constants.CUR_USER.getId());
//		actionLog.setOperator(Constants.CUR_USER.getUserName());
//		actionLog.setLogTime(NowTimeUtils.getNowDate());
//		return actionLog;
//	}
}
