package com.gx.utils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;

import com.gx.obe.message.Messages;
import com.gx.obe.web.service.ConnectionService;

/**
 * @Description: 获取当前时间
 * @author guoyr
 */
public class BusinessNowTimeUtils {

	private static long NOW_TIME = 0l;
	private static Calendar GET_DATETIME_START_CALLENDAR;// 记录获取当前时间的时间点
	private static final String weak[] = {Messages.Weak_sun, Messages.Weak_mon, Messages.Weak_tues, Messages.Weak_wed, Messages.Weak_thur, Messages.Weak_fri, Messages.Weak_sat};
	private static final String weakDay[] = {Messages.Weak_sunday, Messages.Weak_monday, Messages.Weak_tuesday, Messages.Weak_wednesday, Messages.Weak_thursday, Messages.Weak_friday, Messages.Weak_saturday};
//	private static final String weakEn[] = {"Sun", "Mon", "Tues", "Wed", "Thur", "Fri", "Sat"};
//	private static final String weakEnDay[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	private static Logger LOG = Logger.getLogger(BusinessNowTimeUtils.class);
	public static String NOW_DATE_SOURCE = "本机时间";
	private static boolean setTime = false;
	private static ConnectionService connectionService = new ConnectionService();
	
	/**
	 * @Description: 设置当前时间
	 * @author guoyr
	 * @param nowDate
	 */
	public static void setNowTime(Date nowDate){
		Calendar cal = Calendar.getInstance();
		
		cal.setTime(nowDate);
		NOW_TIME = cal.getTimeInMillis();
		GET_DATETIME_START_CALLENDAR = Calendar.getInstance();
		
		// 标记为已设置当前时间
		setTime = true;
		
		LOG.info("license设置当前时间为：" + nowDate);
	}
	
	/**
	 * @Description: 获取数据库或时间服务器中的当前时间 
	 * @author guoyr
	 * @return
	 */
	private static long getNowTime() {
		// 如果还未设置当前时间
		if(!setTime){
			return new Date().getTime() ;
		}
		return NOW_TIME + (Calendar.getInstance().getTimeInMillis()-GET_DATETIME_START_CALLENDAR.getTimeInMillis());
	}
	
	/**
	 * @Description: 获得当前时间
	 * @author guoyr
	 * @return
	 */
	public static Date getNowDate(){
		// 如果还未设置当前时间
		if(!setTime){
			return new Date();
		}
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(getNowTime());
		return calendar.getTime();
	}
	
	/**
	 * @Description: 获得当前年份
	 * @author guoyr
	 * @return
	 */
	public static int getYear() {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(getNowDate());
		return calendar.get(Calendar.YEAR);
	}
	
	/**
	 * @Description: 获得当前星期
	 * @author guoyr
	 * @return
	 */
	public static String getWeak(){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(getNowDate());
		int dayOfWeak = calendar.get(Calendar.DAY_OF_WEEK)-1;
		return weak[dayOfWeak];
	}
	/**
	 * @Description: 获得当前星期
	 * @author guoyr
	 * @return
	 */
	public static String getWeakDay(){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(getNowDate());
		int dayOfWeak = calendar.get(Calendar.DAY_OF_WEEK)-1;
		return weakDay[dayOfWeak];
	}
	
	
	/**
	 * @Description: 获得当前日期
	 * @author guoyr
	 * @return
	 */
	public static String getDate(){
		return new SimpleDateFormat("yyyy-MM-dd").format(getNowDate());
	}
	
	/**
	 * @Description: 获得当前时间
	 * @author guoyr
	 * @return
	 */
	public static String getTime(){
		return new SimpleDateFormat("HH:mm:ss").format(getNowDate());
	}
	
	/**
	 * @Description: 获得当前日期和时间
	 * @author guoyr
	 * @return
	 */
	public static String getDateTime(){
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(getNowDate());
	}
	
	/**
	 * @Description: 服务器的当前时间
	 * @author guoyr
	 * @return
	 */
	private static Date getServiceNowTime(){
		Date nowDate = null;
		long startTime = System.currentTimeMillis();    //获取开始时间
		nowDate = connectionService.getServiceDate();
		long endTime = System.currentTimeMillis();    //获取结束时间
		if(null != nowDate){
			long ms = endTime - startTime;
			if(ms > 1000){
				int second= (int) (ms / 1000);
				Calendar calendar = Calendar.getInstance();
				calendar.setTime(nowDate);
				calendar.add(Calendar.SECOND, second);
				nowDate = calendar.getTime();
			}
		}
		return nowDate;
	}
	
	/**
	 * @Description: 获取时间服务器时间
	 * @author guoyr
	 * @return
	 */
//	private static Date getNowDateFormDateServer(){
//		Date nowDate = null;
//		try {
//			String xmlStr = WebServiceUtil.invokeWebservice(PropertiesUtils.getWebserviceProperty(WebserviceConstants.WEBSERVICE_ADDRESS,"") + PropertiesUtils.getWebserviceProperty(WebserviceConstants.NOW_TIME_URL,""), PropertiesUtils.getWebserviceProperty(WebserviceConstants.NOW_TIME_METHOD,""), new Object[]{}, null);
//			if(!"".equals(xmlStr)){
//				Document document = DocumentHelper.parseText(xmlStr);
//				Element root = document.getRootElement();
//				String operTag = root.element("header").elementTextTrim("operTag");
//				if(null != operTag && "Y".equals(operTag)){
//					String timeStr = root.element("body").elementTextTrim("time");
//					nowDate = DateUtils.toDate(timeStr);
//					LOG.debug("获取时间服务器时间为" + nowDate);
//				}else {
//					LOG.error(root.element("header").elementTextTrim("operDesc"));
//					LOG.error(root.element("header").elementTextTrim("operException"));
//				}
//			}else {
//				LOG.error("获取时间服务器时间出错！");
//			}
//		} catch (Exception e) {
//			LOG.error("", e);
//			nowDate = null;
//		}
//		return nowDate;
//	}
}
