package com.gx.utils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.apache.log4j.Logger;
import org.eclipse.swt.core.calendar.CalendarUtil;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.components.core.Constants;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.web.service.ConnectionService;

/**
 * @Description: 获取当前时间
 * @author guoyr
 */
public class NowTimeUtils {

	private static long NOW_TIME = 0l;
	private static Calendar GET_DATETIME_START_CALLENDAR;// 记录获取当前时间的时间点
	private static final String weak[] = {Messages.Weak_sun, Messages.Weak_mon, Messages.Weak_tues, Messages.Weak_wed, Messages.Weak_thur, Messages.Weak_fri, Messages.Weak_sat};
	private static final String weakDay[] = {Messages.Weak_sunday, Messages.Weak_monday, Messages.Weak_tuesday, Messages.Weak_wednesday, Messages.Weak_thursday, Messages.Weak_friday, Messages.Weak_saturday};
//	private static final String weakEn[] = {"Sun", "Mon", "Tues", "Wed", "Thur", "Fri", "Sat"};
//	private static final String weakEnDay[] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
	private static Logger LOG = Logger.getLogger(NowTimeUtils.class);
	public static String NOW_DATE_SOURCE = "本机时间";
	private static boolean setTime = false;
	private static ConnectionService connectionService = new ConnectionService();
	private static Runnable nowTimeRunnable = new Runnable(){
		public void run() {
			Date nowDate = null;
			//服务器的当前时间
			nowDate = getServiceNowTime();
			if(null != nowDate){
				setNowTime(nowDate);
				NOW_DATE_SOURCE = "服务器时间";
			}
			
			
			
			// 如果需要从时间戳服务器时间获取时间
			if(PropertiesUtils.getProperty(ConfigConstants.NOW_TIME_TYPE, "00").equals("01")){
				Thread getNowDateFormDateServerThread = new Thread(new Runnable(){
					public void run(){
						for(int i = 1; i < 4; i++){
							Date nowDate = getJsonNowDateFormDateServer();
							
							// 如果未能从时间服务器获取到当前时间，最多尝试获取3次
							if(null != nowDate){
								NOW_DATE_SOURCE = "时间戳服务器时间";
								setNowTime(nowDate);
								break;
							}
							LOG.debug("从时间服务器获取到当前时间失败！");
							// 每1分钟获取一次服务器时间
							try {
								Thread.sleep(1000*60*1);
							} catch (InterruptedException e) {
							}
						}
					}
				});
				getNowDateFormDateServerThread.start();
			}
		}
	} ;
	
	/**
	 * @Description: 设置当前时间
	 * @author guoyr
	 * @return
	 */
	public static Thread run(){
		Thread thread = new Thread(nowTimeRunnable);
		thread.start();
		return thread;
	}
	
	/**
	 * @Description: 设置当前时间
	 * @author guoyr
	 * @param nowDate
	 */
	public static void setNowTime(Date nowDate){
		Calendar cal = Calendar.getInstance();
		
		cal.setTime(nowDate);
		NOW_TIME = cal.getTimeInMillis();
		GET_DATETIME_START_CALLENDAR = Calendar.getInstance();
		// 为日历控件设置时间
		CalendarUtil.setNowDate(nowDate);
		// 为license设置当前时间
		com.gx.security.NowTimeUtils.setNowTime(nowDate);
		// 标记为已设置当前时间
		setTime = true;
		
		LOG.info("license设置当前时间为：" + nowDate);
	}
	
	/**
	 * @Description: 获取数据库或时间服务器中的当前时间 
	 * @author guoyr
	 * @return
	 */
	private static long getNowTime() {
		// 如果还未设置当前时间
		if(!setTime){
			return new Date().getTime() ;
		}
		return NOW_TIME + (Calendar.getInstance().getTimeInMillis()-GET_DATETIME_START_CALLENDAR.getTimeInMillis());
	}
	
	/**
	 * @Description: 获得当前时间
	 * @author guoyr
	 * @return
	 */
	public static Date getNowDate(){
		// 如果还未设置当前时间
		if(!setTime){
			return new Date();
		}
		Calendar calendar = Calendar.getInstance();
		calendar.setTimeInMillis(getNowTime());
		return calendar.getTime();
	}
	
	/**
	 * @Description: 获得当前星期
	 * @author guoyr
	 * @return
	 */
	public static String getWeak(){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(getNowDate());
		int dayOfWeak = calendar.get(Calendar.DAY_OF_WEEK)-1;
		return weak[dayOfWeak];
	}
	/**
	 * @Description: 获得当前星期
	 * @author guoyr
	 * @return
	 */
	public static String getWeakDay(){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(getNowDate());
		int dayOfWeak = calendar.get(Calendar.DAY_OF_WEEK)-1;
		return weakDay[dayOfWeak];
	}
	
	
	/**
	 * @Description: 获得当前日期
	 * @author guoyr
	 * @return
	 */
	public static String getDate(){
		return new SimpleDateFormat("yyyy-MM-dd").format(getNowDate());
	}
	
	/**
	 * @Description: 获得当前时间
	 * @author guoyr
	 * @return
	 */
	public static String getTime(){
		return new SimpleDateFormat("HH:mm:ss").format(getNowDate());
	}
	
	/**
	 * @Description: 获得当前日期和时间
	 * @author guoyr
	 * @return
	 */
	public static String getDateTime(){
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(getNowDate());
	}
	
	/**
	 * @Description: 服务器的当前时间
	 * @author guoyr
	 * @return
	 */
	private static Date getServiceNowTime(){
		Date nowDate = null;
		long startTime = System.currentTimeMillis();    //获取开始时间
		nowDate = connectionService.getServiceDate();
		long endTime = System.currentTimeMillis();    //获取结束时间
		if(null != nowDate){
			long ms = endTime - startTime;
			if(ms > 1000){
				int second= (int) (ms / 1000);
				Calendar calendar = Calendar.getInstance();
				calendar.setTime(nowDate);
				calendar.add(Calendar.SECOND, second);
				nowDate = calendar.getTime();
			}
		}
		return nowDate;
	}
	
	/**
	 * 获得24小时后的时间
	 * @return
	 */
	public static Date getTomorrowDate(Date oldDate){
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(oldDate);
		calendar.add(Calendar.DAY_OF_MONTH, 1);
		Date date = calendar.getTime();
		return date;
	}
	
	/**
	 * @Description: 获取时间服务器时间
	 * @author guoyr
	 * @return
	 */
	private static Date getJsonNowDateFormDateServer(){
		Date nowDate = null;
		String method = PropertiesUtils.getWebserviceProperty(WebserviceConstants.NOW_TIME_METHOD,"");
		HttpUrl httpUrl = new HttpUrl(Constants.getPlatformUrl(method));
		httpUrl.setName("获取时间服务器时间");
		try {
			// {"data":{"time":"2017-07-25 09:50:56"},"header":{"ret":"0","msg":"获取时间成功","size":""}}
			HttpResponse httpResponse = HttpUtils.doPost(httpUrl);
//			String xmlStr = WebServiceUtil.invokeWebservice(serverUrl , PropertiesUtils.getWebserviceProperty(WebserviceConstants.NOW_TIME_METHOD,""), new Object[]{}, null);
			if(httpResponse.getStatus()){
				SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				nowDate = format.parse(httpResponse.getDataParam("time"));
				Calendar calendar = Calendar.getInstance();
				// 获得时间后加1秒
				calendar.setTimeInMillis(nowDate.getTime() + 100);
				nowDate = calendar.getTime();
			}
		} catch (Exception e) {
			LOG.error("", e);
			nowDate = null;
		}finally {
			if(null == nowDate) {
				LOG.error("获取时间服务器时间出错！" + httpUrl.getUrl());
			}
		}
		return nowDate;
	}
//	private static Date getNowDateFormDateServer(){
//		Date nowDate = null;
//		try {
//			String xmlStr = WebServiceUtil.invokeWebservice(PropertiesUtils.getWebserviceProperty(WebserviceConstants.WEBSERVICE_ADDRESS,"") + PropertiesUtils.getWebserviceProperty(WebserviceConstants.NOW_TIME_URL,""), PropertiesUtils.getWebserviceProperty(WebserviceConstants.NOW_TIME_METHOD,""), new Object[]{}, null);
//			if(!"".equals(xmlStr)){
//				Document document = DocumentHelper.parseText(xmlStr);
//				Element root = document.getRootElement();
//				String operTag = root.element("header").elementTextTrim("operTag");
//				if(null != operTag && "Y".equals(operTag)){
//					String timeStr = root.element("body").elementTextTrim("time");
//					nowDate = DateUtils.toDate(timeStr);
//					LOG.debug("获取时间服务器时间为" + nowDate);
//				}else {
//					LOG.error(root.element("header").elementTextTrim("operDesc"));
//					LOG.error(root.element("header").elementTextTrim("operException"));
//				}
//			}else {
//				LOG.error("获取时间服务器时间出错！");
//			}
//		} catch (Exception e) {
//			LOG.error("", e);
//			nowDate = null;
//		}
//		return nowDate;
//	}
}
